from typing import List

from pandas import DataFrame

from cadenzaanalytics.data.column_metadata import ColumnMetadata


class ExtensionResponse:
    """A class representing a response from an extension.
    """
    def get_response(self, original_column_metadata: List[ColumnMetadata], original_data: DataFrame):
        """Get the response from the extension.
        """



from typing import Callable, List

from pandas import DataFrame, Series

from cadenzaanalytics.data.column_metadata import ColumnMetadata
from cadenzaanalytics.response.error_response import ErrorResponse
from cadenzaanalytics.response.csv_response import CsvResponse
from cadenzaanalytics.data.attribute_group import KEY_ATTRIBUTE_GROUP_NAME
from cadenzaanalytics.response.extension_response import ExtensionResponse


class RowWiseMappingCsvResponse(ExtensionResponse):
    def __init__(self, response_columns: List[ColumnMetadata], row_mapper: Callable, closure: Callable=None):
        self._response_columns = response_columns
        self._row_mapper = row_mapper
        self._closure = closure

    def get_response(self, original_column_metadata: List[ColumnMetadata], original_data: DataFrame):
        key_attribute_columns = [c for c in original_column_metadata
                                 if c.attribute_group_name == KEY_ATTRIBUTE_GROUP_NAME]
        key_attribute_column_names = [c.name for c in key_attribute_columns]
        truncated_original_data = original_data.drop(key_attribute_column_names, axis=1)
        result_data = truncated_original_data.apply(self._apply_and_wrap, axis=1)
        if len(self._response_columns) != len(result_data.columns):
            return ErrorResponse(
                f"Expected  {len(result_data.columns)} response columns but got {len(self._response_columns)}", 500)
        extended_result_data = result_data.join(original_data.loc[:, key_attribute_column_names])
        csv_response = CsvResponse(extended_result_data, self._response_columns + key_attribute_columns)
        response = csv_response.get_response(original_column_metadata, original_data)
        if self._closure is not None:
            self._closure(result_data)
        return response

    def _apply_and_wrap(self, row):
        mapped_row = self._row_mapper(row)
        if isinstance(mapped_row, Series):
            return mapped_row
        return Series(mapped_row, index=[c.name for c in self._response_columns])



from enum import Enum


class AttributeRole(Enum):
    """A class representing attribute roles in data modeling like dimension and measure.
    """
    DIMENSION = "dimension"
    MEASURE = "measure"

    def __str__(self):
        return self.value



from enum import Enum


# pylint: disable=duplicate-code
class DataType(Enum):
    """A class representing various data types such as string, integer, float, zonedDateTime, and geometry.
    """
    STRING = "string"
    INT64 = "int64"
    FLOAT64 = "float64"
    ZONEDDATETIME = "zonedDateTime"
    GEOMETRY = "geometry"

    def __str__(self):
        return self.value

    def pandas_type(self):
        """Return the corresponding pandas data type for the given value.

        Returns
        -------
        str
            The pandas data type corresponding to the given value.
        """
        if self.value == 'int64':
            return "Int64"
        if self.value == 'float64':
            return "Float64"
        return "string"



from typing import List

from cadenzaanalytics.data.column_metadata import ColumnMetadata
from cadenzaanalytics.data.data_object import DataObject


class DataContainerMetadata(DataObject):
    """A class representing metadata for data containers such as type, name and columns.

    Parameters
    ----------
    DataObject : type
        The base data object type from which DataContainerMetadata inherits.
    """
    _attribute_mapping = {
        "type": "_content_type",
        "name": "_name",
        "columns": "_columns"
    }

    def __init__(self, content_type: str, name: str, columns: List[ColumnMetadata] = None):
        self._content_type = content_type
        self._name = name
        self._columns = columns



import json
from typing import List

from flask import Response
from pandas import DataFrame

from cadenzaanalytics.data.column_metadata import ColumnMetadata
from cadenzaanalytics.response.extension_response import ExtensionResponse


class ErrorResponse(ExtensionResponse):
    """A class representing an error response from an extension.

    Parameters
    ----------
    ExtensionResponse : type
        The base extension response type from which ErrorResponse inherits.
    """
    def __init__(self, message: str, status: int = 400):
        self._message = message
        self._status = status

    def get_response(self, original_column_metadata: List[ColumnMetadata], original_data: DataFrame):
        """Get the error response.

        Returns
        -------
        Response
            The error response.
        """
        return self._create_response(self._message)

    def _get_response_json(self, message: str):
        error_message = {
            "message": message
        }

        return json.dumps(error_message, default=str)

    def _create_response(self, message: str):
        return Response(response=self._get_response_json(message), status=self._status, mimetype="application/json")
